#include "register.h"
#include "hw-reg.h"

#define HPOS 688

#---------- WaitGPU - waits until GPU ready to receive commands

			.global	WaitGPU
			
WaitGPU:    li 		t0,0x10000000
			li		t2,0x1f800000

WaitGPU1:	lw 		t1,GPU1(t2)
            and 	t1,t1,t0
            beqz 	t1,WaitGPU1
            jr 		ra



#---------- WaitIdle - waits until GPU is idle      
			
			.global	WaitIdle

WaitIdle:   li 		t1,0x04000000
			li		t2,0x1f800000
WaitIdle1:  lw 		t0,GPU1(t2)
            and 	t0,t0,t1
            beqz 	t0, WaitIdle1
            jr 		ra

#----------- InitGPU - basic GPU init routine ----------------------------------

#in: a0: display mode

			.global	InitGPU

InitGPU:    subu	sp,sp,8
			sw		ra,(sp)
			sw		s0,4(sp)
			
			li		s0,0x1f800000
            sw 		zero,GPU1(s0)       # reset

            li 		t3,0x03000001
            sw 		t3,GPU1(s0)         # disable display

            li 		t3,0x05000000		# display offset
            sw 		t3,GPU1(s0)

#            li 		t3,0x06c40240       # horizontal start/end    
            li		t3,HPOS|((HPOS+2560)<<12)|0x06000000
            sw 		t3,GPU1(s0)

            li 		t3,0x07049025    	# vertical start/end
            sw 		t3,GPU1(s0)

            li 		t3,0xe1000685    	# draw mode, texture page = (8bit,320,0)
            jal 	WaitGPU
            sw 		t3,GPU0(s0)

            jal 	WaitGPU
            li 		t3,0xe3000000      	# clip start
            sw 		t3,GPU0(s0)

            li 		t3,0xe4000000+(512<<10)+640       # clip w, h
            jal 	WaitGPU
            sw 		t3,GPU0(s0)

            li 		t3,0xe5000000       # draw offset
            jal 	WaitGPU
            sw 		t3,GPU0(s0)

            sw 		a0,GPU1(s0)          # set display mode
            nop

            li 		t3,0x03000000
            sw 		t3,GPU1(s0)          # enable display

			lw		ra,(sp)
			lw		s0,4(sp)
			addu	sp,sp,8
            jr 		ra



#---------- MEM2VRAM_IO - transfer graphic data to VRAM (IO mode)
# in: a0: source address
#     a1: x,y    (y << 16 | x)
#     a2: w,h    (h << 16 | w)
#     a3: amount to copy (in words)

			.global	mem2vram

mem2vram:   move 	t3,ra
            jal 	WaitIdle
            li		t4,0x1f800000
            li 		t2,0x04000000
            sw 		t2,GPU1(t4)
            li 		t0,0x01000000
            li 		t1,0xa0000000
            sw 		t0,GPU1(t4)
            sw 		t1,GPU0(t4)
            sw 		a1,GPU0(t4)
            sw 		a2,GPU0(t4)

            move 	t0,a3           	# anzahl words
            move 	t2,a0               # source

m2v_io_loop:lw 		t1,(t2)  			# get source word
            subu 	t0,t0,1             # i--;
            sw 		t1,GPU0(t4)         # write to vram
            addu 	t2,t2,4             # next word
            bne 	zero,t0,m2v_io_loop

            move 	ra,t3
            jr 		ra
            

#---------- SendList - sends a list of primitives to GPU
# in: a0: address of list
			
			.global	SendList
			
SendList:   move 	t9,ra

            jal 	WaitGPU

			li		t4,0x1f800000
            li 		t2,0x04000002

            lw 		t3,DPCR(t4)
#            sw 		zero,DICR(t4)
            ori		t3,0x800
            sw 		t3,DPCR(t4)

            sw 		t2,GPU1(t4)
            sw 		a0,D2_MADR(t4)
            sw 		zero,D2_BCR(t4)
            li 		t1,0x01000401
            sw 		t1,D2_CHCR(t4)
           
           	jr 		t9



			.global	int_on
			
int_on:		li		a1,0x1f800000
			lw		v0,0x1074(a1)
			nop
			or		a2,v0,a0
			sw		a2,0x1074(a1)
			jr		ra
			
			.global	int_off
			
int_off:	li		a1,0x1f800000
			lw		v0,0x1074(a1)
			nor		a0,a0,zero
			and		a2,v0,a0
			sw		a2,0x1074(a1)
			jr		ra
				

			.global	xprintf

xprintf:	addi    $10,$0,0xa0
		  	addi	$9,$0,0x3f
			jr      $10

			.global	OpenEvent
			
OpenEvent:	addi    $10,$0,0xb0
		  	addi	$9,$0,0x08
			jr      $10

			.global	EnableEvent
			
EnableEvent:addi    $10,$0,0xb0
		  	addi	$9,$0,0x0c
			jr      $10

			.global	CloseEvent
			
CloseEvent:	li		$10,0xb0
			li		$9,9
			jr		$10

			.global	DisableEvent
			
DisableEvent: li	$10,0xb0
			li		$9,0x0d
			jr		$10


			.global	exit_critic

exit_critic:li		a0,2                    	# exit critical section
			syscall	0
			jr		ra

			.global	SetRCnt
			
SetRCnt:	addi    $10,$0,0xb0
		  	addi	$9,$0,0x02
			jr      $10
						

			
			.global	_96_init
			
_96_init:	li		$10,0xa0
			li		$9,0x54
			jr		$10


			.global	_96_remove
_96_remove:	li		$10,0xa0
			li		$9,0x56
			jr		$10


			.global	CDInstallInt
			
CDInstallInt:
			li		$10,0xa0
			li		$9,0xa2
			jr		$10

			.global	LoadCDSec
			
LoadCDSec:	li		$10,0xa0
			li		$9,0xa5
			jr		$10
							

			.global	SeekCD
			
SeekCD:		li		$10,0xa0
			li		$9,0x78
			jr		$10
			
			.global	LoadCD
			
LoadCD:		li		$10,0xa0
			li		$9,0x7e
			jr		$10
						

			.global	SysError
			
SysError:	li		$10,0xa0
			li		$9,0xa1
			jr		$10
							

# char *strcat(char *dst , char *src)

			.global	strcat

strcat:		li		$10,0xa0
			li		$9,0x15
			jr		$10
			

# char *strncat(char *dst , char *src , int n)

			.global	strncat

strncat:	li		$10,0xa0
			li		$9,0x16
			jr		$10

# int strncmp(char *dst , char *src , int n)

			.global	strncmp

strncmp:	li		$10,0xa0
			li		$9,0x18
			jr		$10


			.global	strcmp

strcmp:		li		$10,0xa0
			li		$9,0x17
			jr		$10
			

# char *strncpy(char *dst , char *src , int n))

			.global	strncpy

strncpy:	li		$10,0xa0
			li		$9,0x1a
			jr		$10
			
			.global	AddQueue
			
AddQueue:	li		$10,0xc0
			li		$9,0x02
			jr		$10								